//==========================================================================
// MAINWND.CPP     Source file for the main window functions of the 
//                 MLIGHTS.EXE program.
//
// CREATED BY      George C Cross, for Synergy Solutions Inc
// CREATED ON      August 5, 1995
// LAST MODIFIED   October 3, 1995.  Final build for beta release.
//
// (c) Copyright Synergy Solutions Inc, 1995. All rights reserved.
//==========================================================================

#include "mainwnd.h"   //Header for this source file
#include "mlights.h"   //Header for constant variables
#include "hookfns.h"   //Header for the HOOKFNS.CPP source file
#include "prochook.h"  //Header for the PROCHOOK.LIB library
#include "cshape.h"    //Header for the CShape object
#include "clabel.h"    //Header for the CLabel object
#include "dialog.h"    //Header for the DIALOG.CPP source file 
#include "resource.h"  //Header for resource constants
#include "ctl3d.h"     //Header for Microsoft 3D controls


//==========================================================================
// Below are arrays of constants used to reduce the amount code when
// accomplishing various initialization tasks.
//==========================================================================
                                                   
extern char OptionEntryStrings[NUM_OPTIONS][20];
extern UINT OptionConstants[NUM_OPTIONS];                                      
                                                   
//==========================================================================
// Global variable re declarations
//==========================================================================

extern char __far  glpszMLightsIniPath[60];    //Path to MLIGHTS.INI
extern HINSTANCE   hInst;       //Handle to the data segment of this instance
extern HWND        ghWnd;       //Handle to the main window
extern NPHOOKCHILD                             //Array of near pointers to 
       npHookedFunctions[NUM_HOOKEDFUNCTIONS]; //hooked function structures
extern CShape Lights[NUM_LIGHTS];              //Modem light window objects
extern CLabel LightLabels[NUM_LIGHTS];         //Modem light label objects
extern UINT       gEventWord;        //Communications device event word
                                     //from GetCommEventMask. See HOOKFNS.CPP 
extern int        gPortID;           //Comm device ID returned from OpenComm.
                                     //See HOOKFNS.CPP
extern HFONT      ghLabelFont;       //Handle to font to use for light labels
extern BOOL       gbPortOpened;      //Flag indicating port is open or closed
extern UINT       gwOptions;    //Flag indicating the window's options
extern UINT       gwNewOptions; //Flag indicating change in  window's options have

extern COMSTAT gComStatBuf;   //Windows API structured variable containing 
                              //status information of a communications device
extern COLORREF   gdwLightColorOn;  //Color of lights when they are "ON"
extern COLORREF   gdwLightColorOff; //Color of lights when they are "OFF"
extern BOOL       gbRemoveCaptionOnRestore;//Flag to remove caption on restore
                                           //of the window (SW_RESTORE)

//==========================================================================
// Variable declarations for variables only visible to this module
//==========================================================================

BOOL  gbMouseCaptured;    //Flag indicating whether the mouse is captured
POINT gptMouseHit;        //Point in window where mouse was clicked down                   


//==========================================================================
// Function declarations for functions only visible to this module
//==========================================================================

static void InstallAPIFunctionHooks ();
static void RemoveAPIFunctionHooks ();  
static void OnPaint ();
static void Add3DBorder (HWND hWnd);
LPBYTE GetCommMSRShadow(short idComDev); 
static void OnCreate ();  
static void OnDestroy ();
static void OnLButtonDown (LPARAM lParam);
static void OnMouseMove (LPARAM lParam);
static void OnLButtonUp ();
static void OnLButtonDblClk ();
static void RemoveCaption (HWND hWnd);
static BOOL CheckForChanges ();
static void OnWindowMinimize ();                               
static void OnWindowRestore ();

                               
//==========================================================================
//
//  FUNCTION: MainWndProc(HWND, UINT, WPARAM, LPARAM)
//
//  PURPOSE:  Processes messages
//
//  MESSAGES:
//
//  WM_COMMAND    - application menu (About dialog box)
//  WM_DESTROY    - destroy window
//
//  COMMENTS:
//
//  To process the IDM_ABOUT message, call MakeProcInstance() to get the
//  current instance address of the About() function.  Then call Dialog
//  box which will create the box according to the information in your
//  generic.rc file and turn control over to the About() function.  When
//  it returns, free the intance address.
//
//==========================================================================
long CALLBACK __export MainWndProc (HWND   hWnd,   UINT   message, 
                                    WPARAM wParam, LPARAM lParam)             
{
 switch (message)
    {
     case WM_CREATE:
         OnCreate ();                //Just initializes the mouse captured
         InstallAPIFunctionHooks (); //Set up hooks to our new functions
         break;

     case WM_LBUTTONDOWN: 
         OnLButtonDown (lParam);     //Set mouse capture
         break; 
         
     case WM_MOUSEMOVE:
         OnMouseMove (lParam);       //Moves the MLights window
         break;
      
     case WM_LBUTTONUP:
         OnLButtonUp ();             //Release mouse capture
         break;    
             
     case WM_LBUTTONDBLCLK:          
         OnLButtonDblClk ();         //Show the options dialog
         break;
     
     case WM_PAINT:
         OnPaint ();
         break;
         
     case WM_COMMNOTIFY:
     case WM_TIMER:
         OnCommNotify ();            //Update the modem lights 
         break;
     
     case WM_SIZE:
         switch (wParam)
            {
             case SIZE_MINIMIZED:
                 OnWindowMinimize ();
                 break;
             case SIZE_RESTORED:
                 OnWindowRestore ();
                 break;
             default:
                 return DefWindowProc (hWnd, message, wParam, lParam);
            }            
         break;
        
     case WM_SYSCOLORCHANGE:
         Ctl3dColorChange ();
         break;
         
     case WM_DESTROY:          
         OnDestroy ();
         break;
 
     default:                        // Passes message on if unproccessed    
         return (DefWindowProc (hWnd, message, wParam, lParam));
    }
 return (NULL);
}


//==========================================================================
// FUNCTION        OnCreate ()
//
// PARAMETERS      none
// 
// RETURNS         nothing
//
// REMARKS         Main window initialization stuff
//==========================================================================
void OnCreate ()
{
 gbMouseCaptured = FALSE;
}


//==========================================================================
// FUNCTION        InstallAPIFunctionHooks ()
//
// PARAMETERS      none
//
// RETURNS         nothing
//
// REMARKS         Installs the custom functions which will be called when
//                 another application calls the indicated API functions.
//=========================================================================
static void InstallAPIFunctionHooks ()
{
 FARPROC lpfnMLightsFunction;       //General purpose function pointer

 //
 //Since the SetProcAddress function can be called from the context
 //of any function, "smart callbacks" or "protected mode callbacks" 
 //can not be used and an instance pointer to the new function must 
 //be obtained. First, get an instance pointer to our MOpenComm ()
 //
 lpfnMLightsFunction = MakeProcInstance ((FARPROC) MOpenComm, hInst);
 //
 //Hook into OpenComm API function
 //
 npHookedFunctions[OPENCOMM] = SetProcAddress ((FARPROC) OpenComm,
                                               lpfnMLightsFunction,
                                               FALSE);
                                                                        

 //
 //Get an instance pointer to our MCloseComm () function
 // 
 lpfnMLightsFunction = MakeProcInstance ((FARPROC) MCloseComm, hInst);
 //
 //Hook into CloseComm API function
 //
 npHookedFunctions[CLOSECOMM] = SetProcAddress ((FARPROC) CloseComm,
                                                lpfnMLightsFunction,
                                                FALSE);

/*

////////////////////////////////////////////////////////////////////////////
//   WE ARE NOT GOING TO HOOK "GetCommError" AT THIS STAGE OF DEVELOPMENT
//
//   George Cross    August 1995
////////////////////////////////////////////////////////////////////////////
 
 //
 //Get an instance pointer to our MGetCommError () function
 //
 lpfnMLightsFunction = MakeProcInstance ((FARPROC) MGetCommError, hInst);
 //
 //Hook into GetCommError API function
 //
 npHookedFunctions[GETCOMMERROR] = SetProcAddress ((FARPROC) GetCommError,
                                                   lpfnMLightsFunction,
                                                   FALSE);
*/

 //
 //Get an instance pointer to our MReadComm () function
 //
 lpfnMLightsFunction = MakeProcInstance ((FARPROC) MReadComm, hInst);
 //
 //Hook into ReadComm API function
 //
 npHookedFunctions[READCOMM] = SetProcAddress ((FARPROC) ReadComm,
                                               lpfnMLightsFunction,
                                               FALSE);


 //
 //Get an instance pointer to our MWriteComm () function
 //
 lpfnMLightsFunction = MakeProcInstance ((FARPROC) MWriteComm, hInst);
 //
 //Hook into WriteComm API function
 //
 npHookedFunctions[WRITECOMM] = SetProcAddress ((FARPROC) WriteComm,
                                               lpfnMLightsFunction,
                                               FALSE);


 //
 //Get an instance pointer to our MSetCommEventMask () function
 //
 lpfnMLightsFunction = MakeProcInstance ((FARPROC) MSetCommEventMask, hInst);
 //
 //Hook into SetCommEventMask API function
 //
 npHookedFunctions[SETCOMMEVENTMASK] = SetProcAddress (
                                               (FARPROC) SetCommEventMask,
                                               lpfnMLightsFunction,
                                               FALSE);
}

/*
//==========================================================================
//A macro I decided not to use at this time

#define HOOK_API_FUNCTION(x)  \
        npHookedFunctions[x] =   \
        SetProcAddress ((FARPROC) APIFunctions[x],  \
                        MakeProcInstance ((FARPROC) MLightsFns[x], hInst), \
                        FALSE)
                        
//==========================================================================
*/                                                                          


//==========================================================================
// FUNCTION        RemoveAPIFunctionHooks ()
//
// PARAMETERS      none
//
// RETURNS         nothing
//
// REMARKS         Unhooks all previously hooked functions
//========================================================================= 
static void RemoveAPIFunctionHooks ()  
{
 //
 //For each hooked function,
 //
 for (int iloop = 0; iloop < NUM_HOOKEDFUNCTIONS; iloop++)
    {
     //
     //Unhook it permanently
     //
     SetProcRelease (npHookedFunctions[iloop]);
    }
} //End  RemoveAPIFunctionHooks ()


//==========================================================================
// FUNCTION        OnPaint ()
//
// PARAMETERS      none
//
// RETURNS         nothing
//
// REMARKS         Paints the main window including the lights.
//==========================================================================
void OnPaint()
{
 PAINTSTRUCT ps;   //Windows API structured variable for painting
 HDC hdc;          //Handle to the device context

 hdc = BeginPaint(ghWnd, &ps);  //Must be called before painting

 //
 //Paint all the lights 
 //
 for (int iloop = 0; iloop < NUM_LIGHTS; iloop++)
    {
     Lights[iloop].DrawShape (hdc);
    } 
 
 //
 //And paint labels for them also
 //
 for (iloop = 0; iloop < NUM_LIGHTS; iloop++)
    {
     LightLabels[iloop].DrawLabel (hdc, ghLabelFont);
    } 
 
 //
 //Add 3D look to the main window borders
 //
 Add3DBorder (ghWnd);
 
 
 EndPaint(ghWnd, &ps);          //Must be called after painting
}


//==========================================================================
// FUNCTION        OnCommNotify ()
//
// PARAMETERS      none
//
// RETURNS         nothing
//
// REMARKS         Determines the current communcations device status and 
//                 adjusts each modem "LED light" accordingly. 
//                 The communications device status is determined different
//                 ways depending on the LED light for which we are checking.
//
//                 LED light  Method to determine status
//                 ---------  --------------------------
//
//                 TX         The event word returned by GetCommEventMask()
//
//                 RX         The cbInQue member of the COMSTAT structure
//                            filled by GetCommError()
//
//                 RLSD
//                 CTS
//                 DSR
//                 RI         The MSRShadow byte located 35 bytes past
//                            the event word pointed to by SetCommEventMask()
//                            Please consult Microsoft Knowledge Base article
//                            Q101417.
//                
//==========================================================================
void OnCommNotify ()
{
 //
 //We are calling this function very frequently, so, let's just allocate 
 //the variables once and avoid having to allocate them repetetively.
 //(Hence the "static" modifier)
 //
 static UINT ImmediateEventWord;  //Event word as returned by GetCommEventMask()
 static HDC  hdc;                 //General purpose device context handle

 //
 //Proceed no further with handling the event if the port has been closed.
 //(I had to put this in because CloseComm() function was returning before
 // the WM_COMMNOTIFY message had been completely handled.)
 //
 if (gbPortOpened == FALSE) return;
 
 //
 //Retrieve the current communications device event word
 //
 ImmediateEventWord = GetCommEventMask (gPortID, EV_TXEMPTY);          
 
 //
 //If a byte has been placed in the receive queue, then
 //
 if (ImmediateEventWord & EV_TXEMPTY)
    {
     Lights[TX].Select (FALSE);  //Deselect the object emulating the TX light
     hdc = GetDC (ghWnd);        //Get a handle to the object's device context
     Lights[TX].DrawShape (hdc); //Redraw the object emulating the TX light
     ReleaseDC (ghWnd, hdc);     //Release the handle to the device context
    }

 //
 //Retrieve a buffer filled with status information for this 
 //communcations device
 //
 GetCommError (gPortID, &gComStatBuf);
 
 //
 //If there are no characters in the receive queue and the RX light is ON,
 // 
 if ((gComStatBuf.cbInQue == 0) && (Lights[RX].Selected () == TRUE))
    {
     Lights[RX].Select (FALSE);  //Deselect the object emulating the RX light
     hdc = GetDC (ghWnd);        //Get a handle to the object's device context
     Lights[RX].DrawShape (hdc); //Redraw the object emulating the RX light
     ReleaseDC (ghWnd, hdc);     //Release the handle to the device context
    }

 //
 //Now check status of other modem registers...
 //                                         
 //Get a pointer to the reflection of the modem status register maintained
 //within COMM.DRV at the extended device control block. See GetCommMSRShadow
 //
 LPBYTE lpMSRShadow =  //GetCommMSRShadow (gPortID);
 (LPBYTE)SetCommEventMask (gPortID, 0) + COMM_MSRSHADOW;
 
 //
 //If the RLSD bit is not the same as the "selected" member of the
 //light structure, then,
 //
 if (((*lpMSRShadow) & MSR_RLSD) != Lights[RLSD].Selected ())
    {
     //
     //Set the light to ON or OFF according to what the RLSD bit in the
     //COMM.DRV modem status register reflects
     //
     Lights[RLSD].Select ((*lpMSRShadow) & MSR_RLSD); 
     hdc = GetDC (ghWnd);                    //Get window device context
     Lights[RLSD].DrawShape (hdc);           //Draw the light ON or OFF
     ReleaseDC (ghWnd, hdc);                 //Release the device context
    }
     
 //
 //Check RI bit same procedure as for RLSD above
 //
 if ((*lpMSRShadow) & MSR_RI != Lights[RI].Selected ())
    {
     Lights[RI].Select ((*lpMSRShadow) & MSR_RI);
     hdc = GetDC (ghWnd);
     Lights[RI].DrawShape (hdc);
     ReleaseDC (ghWnd, hdc);
    }
     
 //
 //Check the DSR bit same procedure as for RLSD above
 //
 if (((*lpMSRShadow) & MSR_DSR) != Lights[DSR].Selected ())
    {
     Lights[DSR].Select ((*lpMSRShadow) & MSR_DSR);
     hdc = GetDC (ghWnd);
     Lights[DSR].DrawShape (hdc);
     ReleaseDC (ghWnd, hdc);
    }
     
 //
 //Check the CTS bit same procedure as for RLSD above
 //
 if (((*lpMSRShadow) & MSR_CTS) != Lights[CTS].Selected ())
    {
     Lights[CTS].Select ((*lpMSRShadow) & MSR_CTS);
     hdc = GetDC (ghWnd);
     Lights[CTS].DrawShape (hdc);
     ReleaseDC (ghWnd, hdc);
    }

      
} //End  void OnCommNotify () 

 
//==========================================================================
// FUNCTION        Add3DBorder (HWND hWnd);
//
// PARAMETERS      hWnd     Handle to the window to which a 3D border is to
//                          be added
//
// RETURNS         nothing
//
// REMARKS         Draws a 1 pixel width white line on left and top. Draws
//                 a 2 pixel width gray line on right and bottom.
//==========================================================================
void Add3DBorder (HWND hWnd)
{
 RECT        rc;               //Rectangle to hold coordinates of client area
 HPEN        hPen, hOldPen;    //Handles to pens used to draw the 3D effects
 HDC         hdc;              //Handle to the window device context

 //
 //Get the coordinates of the main window client area
 GetClientRect(hWnd, &rc);
 
 //
 //Get a device context for the window
 // 
 hdc = GetDC (hWnd);
 
 //
 //Create a white pen and select it in to the window's device context
 //
 hPen = CreatePen (PS_SOLID, 1, GetSysColor (COLOR_BTNHIGHLIGHT));
 hOldPen = SelectObject (hdc, hPen);
 
 //
 //Draw a white line on left and top sides
 //
 MoveTo (hdc, rc.left, rc.bottom);
 LineTo (hdc, rc.left, rc.top);
 LineTo (hdc, rc.right, rc.top);
 
 //
 //Remove the pen from the device context and free its from GDI resources
 //
 SelectObject (hdc, hOldPen);
 DeleteObject (hPen);
                                                      
 //
 //Create a shadow pen and select it into the window's device context
 //                                                      
 hPen = CreatePen (PS_SOLID, 2, GetSysColor (COLOR_BTNSHADOW));
 hOldPen = SelectObject (hdc, hPen); 
 
 //
 //Draw a shadow line on right and bottom sides
 // 
 LineTo (hdc, rc.right, rc.bottom);
 LineTo (hdc, rc.left, rc.bottom);
   
 //
 //Remove the pen from the device context and free its from GDI resources
 //
 SelectObject (hdc, hOldPen);
 DeleteObject (hPen);
 
 //
 //Release the device context
 //
 ReleaseDC (hWnd, hdc);
}    


//==========================================================================
// FUNCTION       OnLButtonDown ()
//
// PARAMETERS      lParam    LOWORD contains horizontal position of cursor
//                           HIWORD contains vertical position of cursor
//
// RETURNS         nothing
//
// REMARKS         Captures the mouse movement if not already captured
//==========================================================================
void OnLButtonDown (LPARAM lParam)
{
 //
 //If the mouse is not already captured, then,
 //
 if (gbMouseCaptured == FALSE)
    {
     //
     //Capture the mouse's movements
     //
     SetCapture (ghWnd);
     
     //
     //Record the point at which the mouse is commencing dragging
     //
     gptMouseHit.x = LOWORD(lParam);
     gptMouseHit.y = HIWORD(lParam);
     
     //
     //Raise the capture flag
     //
     gbMouseCaptured = TRUE;
    }  //End   if (gbMouseCaptured == FALSE)
} //End   void OnLButtonDown (LPARAM lParam)


//==========================================================================
// FUNCTION        OnMouseMove ()
//
// PARAMETERS      lParam    LOWORD contains horizontal position of cursor
//                           HIWORD contains vertical position of cursor
//
// RETURNS         nothing
//
// REMARKS         Moves the window accordingly if the left button is down
//==========================================================================
void OnMouseMove (LPARAM lParam)
{
 //
 //If the left mouse button is down, then the capture flag will be TRUE, so,
 //
 if (gbMouseCaptured == TRUE)
    {
     //
     //Save the current mouse position as a POINT variable
     //
     POINT pt;
     pt.x = LOWORD(lParam);
     pt.y = HIWORD(lParam);
     
     //
     //Convert the current mouse position to screen coordinates
     //
     ClientToScreen (ghWnd, &pt); 
     
     //
     //Move the window to its new position
     //
     SetWindowPos (ghWnd,
                   NULL,
                   pt.x - gptMouseHit.x,
                   pt.y - gptMouseHit.y,
                   0,
                   0,
                   SWP_NOSIZE | SWP_NOZORDER);
   } //End  if (gbMouseCaptured == TRUE)
}  // End  void OnMouseMove (LPARAM lParam)                 
     

//==========================================================================
// FUNCTION        OnLButtonUp ()
//
// PARAMETERS      None
//
// RETURNS         Nothing
//
// REMARKS         Releases the captured mouse if necessary, thus terminating
//                 window dragging.
//==========================================================================
void OnLButtonUp ()
{
 //
 //If the mouse is presently captured, then
 //
 if (gbMouseCaptured == TRUE)
    {
     //
     //Release the mouse
     //
     ReleaseCapture ();
     
     //
     //Lower the flag indicating the mouse is captured
     //
     gbMouseCaptured = FALSE;
    }  //End   if (gbMouseCaptured == TRUE)
} //End  void OnLButtonUp ()   
 
 
//==========================================================================
// FUNCTION        OnLButtonDblClk ()
//
// PARAMETERS      none
//
// RETURNS         nothing
//
// REMARKS         Displays the Modem Lights Options dialog
//==========================================================================
void OnLButtonDblClk ()
{
 FARPROC lpDialogProc;   //(__far __pascal*) (void). Function pointer to 
                         //the Options dialog message-handling procedure
 //
 //Get a pointer to this instance's use of DialogProc
 //
 lpDialogProc = MakeProcInstance((FARPROC)DialogProc, hInst);

 //
 //Invoke the dialog
 //
 DialogBox(hInst,
           MAKEINTRESOURCE(IDD_MLIGHTSDIALOG),
           ghWnd,
           lpDialogProc);
 //
 //Delete the prolog code for the DialogProc function
 //
 FreeProcInstance(lpDialogProc);
 
 //
 //Compare flags for any changes
 //
 CheckForChanges ();

} //End  void OnLButtonDblClk ()


//==========================================================================
// FUNCTION        CheckForChanges ()
//
// PARAMETERS      none
//
// RETURNS         BOOL      TRUE if there were changes. FALSE otherwise.
//
// REMARKS         Checks for changes in the options word and calls functions
//                 appropriately to make adjustments
//==========================================================================
BOOL CheckForChanges ()
{
 int returnval;  //Value which this function will return (TRUE|FALSE)
 //
 //If there were changes
 //
 if (gwOptions != gwNewOptions)
    {
     //
     //If the COLORSCHANGED flag was raised. Note that this flag is not
     //a static indicator such as MINIFPORTCLOSED, TOPMOST and CAPTIONOFF are.
     //
     if (gwNewOptions & COLORSCHANGED)
        {
         //
         //Lower the COLORSCHANGED flag
         //
         gwNewOptions = gwNewOptions & ~(COLORSCHANGED);
         
         //
         //For each modem light
         //
         for (int iloop = 0; iloop < NUM_LIGHTS; iloop++)
            {
             //
             //Reset the color
             //
             Lights[iloop].SetColors (gdwLightColorOn, gdwLightColorOff);                  //Assign position
             
             //
             //Invalidate the window elements which emulate lights
             //
             InvalidateRect (ghWnd, &Lights[iloop], FALSE);
            }  //End   for (int iloop = 0; iloop < NUM_LIGHTS; iloop++)
         
         //
         //Repaint the window for color changes to take effect
         //
         UpdateWindow (ghWnd);
        } // End    if (gwNewOptions & COLORSCHANGED)  
         
     //
     //If the HIDECAPTION bit flag changed, then
     //
     if ((gwOptions ^ gwNewOptions) & HIDECAPTION)
        {
         //
         //If the HIDECAPTION bit flag changed to "On", then
         //
         if (gwNewOptions & HIDECAPTION) 
            {
             //
             //Hide the caption
             //
             RemoveCaption (ghWnd);   
            } //End  if (gwNewOptions & HIDECAPTION)
         else  //Else, (gwNewOptions & HIDECAPTION) == FALSE
            {  
             //
             //The HIDECAPTION bit flag changed to "off"
             //
             AddCaption (ghWnd);  //Show the caption
            } //End else 
        } //End if  ((gwOptions ^ gwNewOptions) & HIDECAPTION) == TRUE) 
     
     //
     //If the TOPMOST bit flag changed, then
     //
     if ((gwOptions ^ gwNewOptions) & TOPMOST)
        {
         //
         //If the TOPMOST bit flag changed to "On", then
         //
         if (gwNewOptions & TOPMOST)
            {
             //
             //Put the window permanently on top
             //
             SetWindowPos (ghWnd, HWND_TOPMOST, 0, 0, 0, 0,
                           SWP_NOMOVE | SWP_NOSIZE);
            }  
         else //Else the TOPMOST bit flat changed to "Off", so
            {
             //
             //Remove the window's TOPMOST status
             //
             SetWindowPos (ghWnd, HWND_NOTOPMOST, 0, 0, 0, 0, 
                           SWP_NOMOVE | SWP_NOSIZE);   
            } //End else
        } //End  if (((gwOptions ^ gwNewOptions) & TOPMOST) == TRUE)    

     //
     //Set the new options word accordingly
     //
     gwOptions = gwNewOptions;   
       
     returnval = TRUE;    //There were changes in the options word
    } //End  if (gwOptions != gbNewOptions)
 else  //Else there were no changes in the options word
    {
     returnval = FALSE;   //No changes in the options word
    }

 //
 //Minimize the window if the MINIFPORTCLOSED bit flag is on in the
 //options word AND the port is not open
 //            
 if ((gwOptions & MINIFPORTCLOSED) && (!(gbPortOpened)))
    {
     if (gwOptions & HIDECAPTION)
        {
         AddCaption (ghWnd);
        } 
     ShowWindow (ghWnd, SW_SHOWMINNOACTIVE);  //Minimize it !
     gbRemoveCaptionOnRestore = TRUE;
    }

 return returnval;   
} //End   BOOL CheckForChanges ()


//==========================================================================
// FUNCTION        AddCaption()
//
// PARAMETERS      hWnd      Handle to the window of which we want to show
//                           the caption.
//                 hMenu     Handle to the window's menu
//
// RETURNS         nothing
//
// REMARKS         Shows the caption and menu bar of a window.  This function
//                 would only be called if the caption and menu were hidden.
//==========================================================================
void AddCaption (HWND hWnd)
{
 DWORD dwStyle;          //Window style
 RECT rc;                //Rectangle holding window dimensions

 //
 //Add the styles which we need for the caption  
 //
 dwStyle = WS_CAPTION | WS_BORDER | WS_SYSMENU | WS_MINIMIZEBOX ;
 
 //
 // Set the new window style
 //
 SetWindowLong (hWnd, GWL_STYLE, dwStyle);
 
 //
 //Get the dimensions of the window
 //
 GetWindowRect (hWnd, &rc); 
 
 //
 //Adjust window height to allow for additional caption and menu 
 //
 rc.top = rc.top - GetSystemMetrics (SM_CYCAPTION);
 
 //
 //Redraw the frame of the window
 //
 SetWindowPos (hWnd, 
               HWND_TOP, 
               rc.left, 
               rc.top,
               rc.right - rc.left,
               rc.bottom - rc.top,
               NULL);
 
 ShowWindow (hWnd, SW_SHOWNORMAL);

} //End  void AddCaption (HWND hWnd)


//==========================================================================
// FUNCTION        RemoveCaption ()
//
// PARAMETERS      hWnd      Handle to the window of which we want to hide
//                           the caption.
//
// RETURNS         hMenu     Handle to the window's menu 
// 
// REMARKS         Hides the caption and menu of a window
//==========================================================================
void RemoveCaption (HWND hWnd)
{
 DWORD dwStyle;           //Window style
 RECT rc1, rc2;           //Rectangle holding window dimensions
 //
 //Hide the window
 //
 ShowWindow (hWnd, SW_HIDE);

 //
 //Change the style to pop up with a border
 //
 dwStyle = WS_POPUP | WS_BORDER;

 // Set the new window style
 SetWindowLong (hWnd, GWL_STYLE, dwStyle);

 //
 //Get the dimensions of the window
 //
 GetWindowRect (hWnd, &rc1); 
 
 //
 //Adjust window height to allow for additional caption and menu 
 //
 rc2.left = rc1.left;
 rc2.top = rc1.top + GetSystemMetrics (SM_CYCAPTION);
 rc2.right = rc1.right;
 rc2.bottom = rc1.bottom;
 
 //
 //Redraw the frame of the window
 //
 SetWindowPos (hWnd, 
               HWND_TOP,
               rc2.left,
               rc2.top,
               rc2.right - rc2.left,
               rc2.bottom - rc2.top,
               NULL);

 ShowWindow (hWnd, SW_SHOWNORMAL); 
}  //End  void RemoveCaption (HWND hWnd, HMENU hMenu) 


//==========================================================================
// FUNCTION        OnWindowMinimize ()
// PARAMETERS      none
// RETURNS         nothing
// REMARKS         Used to ensure the icon is not a top most window
//==========================================================================
void OnWindowMinimize ()
{
 //
 //Minimized windows we don't want TOPMOST
 //
 if (gwOptions & TOPMOST) 
    {
     SetWindowPos (ghWnd, HWND_NOTOPMOST, 0, 0, 0, 0, 
                   SWP_NOMOVE | SWP_NOSIZE);
    }
} //End  void OnWindowMinimize ()
                       

//==========================================================================
// FUNCTION        OnWindowRestore ()
//
// PARAMETERS      none
//
// RETURNS         nothing
//
// REMARKS         Used to ensure the icon is not a top most window
//==========================================================================
void OnWindowRestore ()
{
 DWORD dwStyle = GetWindowLong (ghWnd, GWL_STYLE);
                                
 if ((gbRemoveCaptionOnRestore == TRUE)  && 
     (dwStyle & WS_CAPTION))
    {
     gbRemoveCaptionOnRestore = FALSE;
     if (gwOptions & HIDECAPTION)
        {
         RemoveCaption (ghWnd);
        }
    } //End   if (dwStyle & WS_CAPTION) && . ...ETC. 
          
 //
 //Set the window's style to HWND_TOPMOST if the TOPMOST bit flag is set
 //in the options word
 //
 if (gwOptions & TOPMOST) 
    {
     SetWindowPos (ghWnd, HWND_TOPMOST, 0, 0, 0, 0, 
                   SWP_NOMOVE | SWP_NOSIZE);
    }
} //End  void OnWindowRestore ()
                       

//==========================================================================
// FUNCTION        OnDestroy ()
//
// PARAMETERS      none
//
// RETURNS         nothing
//
// REMARKS         Frees resources allocated for this window and closes 
//                 down the application
//==========================================================================
void OnDestroy ()
{
 RemoveAPIFunctionHooks ();  //Remove hooks to all hooked functions
 DeleteObject (ghLabelFont); //Delete the font we created
 Ctl3dUnregister (hInst);    //Unregister the Microsoft 3D controls

 for (int iloop = 0; iloop < NUM_OPTIONS; iloop++)
    {
     //
     //Remember the state of the main window. If the caption is hidden, then
     //
     if ((gwOptions & OptionConstants[iloop]) == FALSE)
        {
         WritePrivateProfileString ("Options",      //[Options] section
                                    OptionEntryStrings[iloop], //Entry
                                    "Off",           //Assignment to "Caption"
                                    glpszMLightsIniPath); //MLIGHTS.INI file
        }
     else  //The caption is showing
        {
         WritePrivateProfileString ("Options",      //[Options] section
                                    OptionEntryStrings[iloop], //Entry
                                    "On",            //Assignment to "Caption"
                                    glpszMLightsIniPath); //MLIGHTS.INI file
        }
    }//End  for (int iloop = 0; iloop < NUM_OPTIONS; iloop++)
    
 //
 //Record the COLORREF values for the Lights when they are "ON"
 //      
 //Convert the unsigned long value to a character string
 //
 char szString[15];
 wsprintf ((LPSTR) szString, "%lu", gdwLightColorOn);

 //
 //Write that COLORREF value as a string in the MLIGHTS.INI file
 //
 WritePrivateProfileString ("Colors",               //[Colors] section
                            "ColorOn",              //"ColorOn" entry
                            szString,               //COLORREF value
                            glpszMLightsIniPath);   //[path]MLIGHTS.INI
 
 //
 //Write the COLORREF value for the Lights when they are "OFF"
 //
 //Convert the unsigned long value to a character string
 //
 wsprintf ((LPSTR) szString, "%lu", gdwLightColorOff);

 //
 //Write the COLORREF value as a string in the MLIGHTS.INI file
 //
 WritePrivateProfileString ("Colors",               //[Colors] section
                            "ColorOff",             //"ColorOff" entry
                            szString,               //COLORREF value
                            glpszMLightsIniPath);   //[path]MLIGHTS.INI

 //
 //Get the main window's coordinate positions
 //
 RECT r;
 GetWindowRect (ghWnd, &r); 
 
 //
 //Convert the left positions to a character string
 //
 wsprintf ((LPSTR) szString, "%u", r.left);
 
 //
 //Write the main window's left position to MLIGHTS.INI
 //
 WritePrivateProfileString ("Position",          //[Position] section
                            "Left",              //"Left" entry
                            szString,            //Left position of window
                            glpszMLightsIniPath);//[path]MLIGHTS.INI
                            
 //
 //Convert the left positions to a character string
 //
 wsprintf ((LPSTR) szString, "%u", r.top);
 
 //
 //Write the main window's left position to MLIGHTS.INI
 //
 WritePrivateProfileString ("Position",          //[Position] section
                            "Top",               //"Left" entry
                            szString,            //Left position of window
                            glpszMLightsIniPath);//[path]MLIGHTS.INI
 
 
 PostQuitMessage (FALSE);    //Close the application
}

